import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.util.Vector;

public class LotkaVolterraGUI {

    static double ratio1 = 1000;
    static double ratio2 = 100000;

    public static void main(String[] args) {
	JFrame frame = new JFrame("Swing Paint Demo");
	frame.setLayout(new BorderLayout());
	final Modele modele = new Modele();
	final Vue vue = new Vue(modele);
	frame.add(vue);
	JPanel glissieres = new JPanel();
	glissieres.setLayout(new GridLayout(1,6));
	JSlider glissiereX0    = new JSlider(JSlider.VERTICAL, 0, 1000, (int) modele.getX0());
	JSlider glissiereY0    = new JSlider(JSlider.VERTICAL, 0, 1000, (int) modele.getY0());
	JSlider glissiereAlpha = new JSlider(JSlider.VERTICAL, 0, 1000, (int) (ratio1*modele.getAlpha()));
	JSlider glissiereBeta  = new JSlider(JSlider.VERTICAL, 0, 1000, (int) (ratio2*modele.getBeta()));
	JSlider glissiereGamma = new JSlider(JSlider.VERTICAL, 0, 1000, (int) (ratio1*modele.getGamma()));
	JSlider glissiereDelta = new JSlider(JSlider.VERTICAL, 0, 1000, (int) (ratio2*modele.getDelta()));
	glissiereX0.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		modele.setX0( ((JSlider) change.getSource()).getValue());
		vue.update();
	    }
	});
	glissiereY0.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		modele.setY0( ((JSlider) change.getSource()).getValue());
		vue.update();
	    }
	});
	glissiereAlpha.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		modele.setAlpha(((JSlider) change.getSource()).getValue()/ratio1);
		vue.update();
	    }
	});
	glissiereBeta.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		modele.setBeta(((JSlider) change.getSource()).getValue()/ratio2);
		vue.update();
	    }
	});
	glissiereGamma.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		modele.setGamma(((JSlider) change.getSource()).getValue()/ratio1);
		vue.update();
	    }
	});
	glissiereDelta.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		modele.setDelta(((JSlider) change.getSource()).getValue()/ratio2);
		vue.update();
	    }
	});

	glissieres.add(glissiereX0);
	glissieres.add(glissiereY0);
	glissieres.add(glissiereAlpha);
	glissieres.add(glissiereBeta);
	glissieres.add(glissiereGamma);
	glissieres.add(glissiereDelta);

	frame.add(glissieres,BorderLayout.EAST);

	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	frame.pack();
	frame.setVisible(true);
    }
}

class Modele {
    private double x0 = 100;
    private double y0 = 10;
    private double alpha = .3;
    private double beta  = .001;
    private double gamma = .1;
    private double delta = .001;

    public double getX0() {
	return x0;
    }

    public void setX0(double x0) {
	this.x0 = x0;
    }

    public double getY0() {
	return y0;
    }

    public void setY0(double y0) {
	this.y0 = y0;
    }

    public double getAlpha() {
	return alpha;
    }

    public void setAlpha(double alpha) {
	this.alpha = alpha;
    }

    public double getBeta() {
	return beta;
    }

    public void setBeta(double beta) {
	this.beta = beta;
    }

    public double getGamma() {
	return gamma;
    }

    public void setGamma(double gamma) {
	this.gamma = gamma;
    }

    public double getDelta() {
	return delta;
    }

    public void setDelta(double delta) {
	this.delta = delta;
    }

    LotkaVolterraModele lotkaVoltera() {
	return new LotkaVolterraModele(x0, y0, alpha, beta, gamma, delta);
    }
}

class Vue extends JPanel {
    static final int XSCALE=4;
   
    Vector<Double> x;
    Vector<Double> y;
    private Modele modele;
    public Vue(Modele modele) {
	this.modele = modele;
	update();
	setBorder(BorderFactory.createLineBorder(Color.black));
	setBackground(Color.white);
    }

    public void update() {
	repaint();
    }

    public Dimension getPreferredSize() {
	return new Dimension(1000,1000);
    }

    int scalex(double x) {
	return (int) (XSCALE*x);
    }

    int scaley(double y) {
	return (int) (getSize().height-y);
    }

    public void paintComponent(Graphics g) {
	int tmax = getSize().width / XSCALE;
	super.paintComponent(g);
	x = new Vector<Double>();
	y = new Vector<Double>();
	LotkaVolterraModele lotkaVolterra = modele.lotkaVoltera();
	for (int t=0; t<=tmax; t++) {
	    x.add(lotkaVolterra.x);
	    y.add(lotkaVolterra.y);
	    lotkaVolterra.evolue();
	}
	g.drawString("X0:    "+modele.getX0(),    8, 15);
	g.drawString("Y0:    "+modele.getY0(),    8, 30);
	g.drawString("alpha: "+modele.getAlpha(), 8, 45);
	g.drawString("beta:  "+modele.getBeta(),  8, 60);
	g.drawString("gamma: "+modele.getGamma(), 8, 75);
	g.drawString("delta: "+modele.getDelta(), 8, 90);
	for (int t=1; t<tmax; t++) {
	    g.setColor(Color.green);
	    g.drawLine(scalex(t-1), scaley(x.get(t-1)), scalex(t), scaley(x.get(t)));
	    g.setColor(Color.red);
	    g.drawLine(scalex(t-1), scaley(y.get(t-1)), scalex(t), scaley(y.get(t)));
	}
    }
}
